@@ -1,6 +1,6 @@
 ;;;; remote-rep.jl -- Remote file access via the rep-remote program
 ;;;  Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-;;;  $Id: remote-rep.jl,v 1.9 2000/05/15 18:26:49 john Exp $
+;;;  $Id$
 
 ;;; This file is part of librep.
 
@@ -18,13 +18,26 @@
 ;;; along with librep; see the file COPYING.  If not, write to
 ;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(define-structure remote-rep (export remote-rep-add-passwd
-				     remote-rep-close-host
-				     remote-rep-close-all
-				     remote-rep-empty-cache)
-  (open rep remote-utils)
+(declare (unsafe-for-call/cc))
+
+(define-structure rep.io.file-handlers.remote.rep
+
+    (export remote-rep-add-passwd
+	    remote-rep-close-host
+	    remote-rep-close-all
+	    remote-rep-empty-cache)
+
+    (open rep
+	  rep.regexp
+	  rep.system
+	  rep.io.processes
+	  rep.io.files
+	  rep.io.file-handlers
+	  rep.io.file-handlers.remote.utils)
+
+  (define-structure-alias remote-rep rep.io.file-handlers.remote.rep)
+
 
-
 ;; Configuration
 
 ;; rsh doesn't ask for passwords when changing uids,
@@ -70,7 +83,7 @@
 			       (setq i (1+ i)))
 			     map))
 
-
+
 ;; session structure
 
 (defconst remote-rep-host 0)
@@ -89,7 +102,7 @@
   `(eq (aref ,session remote-rep-status) ,stat))
 
 ;; Return an rep structure for HOST and USER, with a running rep session
-(defun remote-rep-open-host (host &optional user)
+(defun remote-rep-open-host (host #!optional user)
   (unless user
     (setq user (remote-get-user host)))
   (catch 'foo
@@ -142,7 +155,7 @@
     (set-process-error-stream (aref session remote-rep-process) nil)
     (kill-process (aref session remote-rep-process))))
 
-(defun remote-rep-close-host (host &optional user)
+(defun remote-rep-close-host (host #!optional user)
   "Close the rep-remote subprocess connected to `USER@HOST'."
   (interactive "sHost:\nsUser:")
   (when (or (null user) (string= user ""))
@@ -167,10 +180,10 @@
 		 (throw 'return s)))
 	  remote-rep-sessions)))
 
-
+
 ;; Communicating with the remote process
 
-(defun remote-rep-write (session fmt &rest args)
+(defun remote-rep-write (session fmt #!rest args)
   (when (remote-rep-status-p session 'dying)
     (error "rep-remote session is dying"))
   (apply format (aref session remote-rep-process) fmt args)
@@ -183,17 +196,18 @@
   (remote-rep-send-int session (length string))
   (remote-rep-write session "%s" string))
 
-(defun remote-rep-while (session status &optional type)
+(defun remote-rep-while (session status #!optional type)
   (when (and (not (eq status 'dying))
 	     (remote-rep-status-p session 'dying))
     (error "rep-remote session is dying"))
   (while (remote-rep-status-p session status)
     (when (and (process-running-p (aref session remote-rep-process))
-	       (accept-process-output remote-rep-timeout))
+	       (accept-process-output-1 (aref session remote-rep-process)
+					remote-rep-timeout))
       (aset session remote-rep-status 'timed-out)
       (error "rep-remote process timed out (%s)" (or type "unknown")))))
 
-(defun remote-rep-command (session type &optional output-fun &rest args)
+(defun remote-rep-command (session type #!optional output-fun #!rest args)
   (when remote-rep-display-progress
     (message (format nil "rep %c %s: " type args) t))
   (remote-rep-while session 'busy type)
@@ -208,10 +222,11 @@
   (remote-rep-error-if-unsuccessful session type args))
 
 ;; Return t if successful, else signal a file-error
-(defun remote-rep-error-if-unsuccessful (session &optional type args)
+(defun remote-rep-error-if-unsuccessful (session #!optional type args)
   (or (eq (aref session remote-rep-status) 'success)
       (signal 'file-error
 	      (list (aref session remote-rep-error)
+		    type
 		    (format nil "%s@%s %s"
 			    (aref session remote-rep-user)
 			    (aref session remote-rep-host) args)))))
@@ -284,8 +299,9 @@
 			 (substring output point))
 		   (setq point (length output)))))
 	      (t
-	       (format standard-error "remote-rep: unhandled output %S\n"
-		       (substring output point))
+;	       (unless (string-looking-at "\\s*$" output point)
+;		 (format standard-error "remote-rep: unhandled output %S\n"
+;			 (substring output point)))
 	       (setq point (length output))))))))
 
 (defun remote-rep-sentinel (process)
@@ -299,7 +315,7 @@
       (aset session remote-rep-callback nil)
       (setq remote-rep-sessions (delq session remote-rep-sessions)))))
 
-
+
 ;; Commands
 
 ;; SESSION has been started, wait for the connection to
@@ -445,7 +461,7 @@
       (aset session remote-rep-callback nil))
     remote-rep-link))
 
-
+
 ;; Directory handling/caching
 
 (defconst remote-rep-file-name 0)
@@ -514,7 +530,7 @@
 	  ;; construct the callback function to have the new cache entry
 	  ;; as the first argument
 	  (aset session remote-rep-callback
-		(lambda (&rest args)
+		(lambda (#!rest args)
 		  (apply remote-rep-dircache-callback entry args)))
 	  (unwind-protect
 	      (condition-case nil
@@ -595,7 +611,7 @@
   (mapc (lambda (ses)
 	  (aset ses remote-rep-dircache nil)) remote-rep-sessions))
 
-
+
 ;; Password caching
 
 (defun remote-rep-get-passwd (user host)
@@ -620,7 +636,7 @@
       (setq remote-rep-passwd-alist (cons (cons joined passwd)
 					  remote-rep-passwd-alist)))))
 
-
+
 ;; Backend handler
 
 (defun remote-rep-handler (split-name op args)
@@ -777,16 +793,16 @@
 	   ((eq op 'file-readable-p)
 	    (and file (/= (logand (aref file remote-rep-file-modes)
 				  (if (remote-rep-file-owner-p session file)
-				      0400 0004)) 0)))
+				      #o400 #o004)) 0)))
 	   ((eq op 'file-writable-p)
 	    (and file (/= (logand (aref file remote-rep-file-modes)
 				  (if (remote-rep-file-owner-p session file)
-				      0200 0002)) 0)))
+				      #o200 #o002)) 0)))
 	   (t
 	    (error "Unsupported rep-remote op: %s %s" op args))))))))))
 
 ;;;###autoload (put 'rep 'remote-backend 'remote-rep-handler)
 
-;;;###autoload (autoload-file-handler 'remote-rep-handler 'remote-rep)
+;;;###autoload (autoload-file-handler 'remote-rep-handler 'rep.io.file-handlers.remote.rep)
 
 (define-file-handler 'remote-rep-handler remote-rep-handler))
