@@ -1,7 +1,7 @@
 ;; tar-file-handler.jl -- pretend that tar files are (read-only) directories
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; $Id: tar.jl,v 1.16 2000/09/10 20:03:17 john Exp $
+;; $Id: tar.jl,v 1.18 2001/03/22 21:43:45 jsh Exp $
 
 ;; This file is part of librep.
 
@@ -31,8 +31,6 @@
 ;; file each time. It would be better to untar the entire contents
 ;; somewhere, and then clean up later..)
 
-;; It needs to use GNU tar (for the compression options)
-
 (declare (unsafe-for-call/cc))
 
 (define-structure rep.io.file-handlers.tar ()
@@ -45,13 +43,21 @@
 	  rep.system
 	  rep.util.date)
 
-
+;; Warning!
+
+;; Before using any more tar options, make sure that the `emulate-gnu-tar'
+;; script can support them.
+
+
 ;; configuration
 
 (defvar tarfh-gnu-tar-program "tar"
   "Location of GNU tar program.")
 
-(defvar tarfh-alternative-gnu-tar-programs '("gtar"))
+(defvar tarfh-alternative-gnu-tar-programs
+  (list "gnutar" "gtar"
+	(expand-file-name
+	 "emulate-gnu-tar" exec-directory)))
 
 ;; Initialised to the current tar version
 (defvar tarfh-gnu-tar-version nil)
@@ -59,11 +65,17 @@
 ;; alist mapping file suffixes to GNU tar compression options
 (defvar tarfh-compression-modes '(("\\.t?gz$" . "--gzip")
 				  ("\\.(taz|Z)$" . "--compress")
-				  ("\\.bz2" . "--bzip2")))
+				  ("\\.bz2" . "--bzip2")
+				  ("\\.xz" . "--xz")
+				  ("\\.(lz|lzma)$" . "--lzma")))
 
 ;; Hairy regexp matching tar `--list --verbose' output
-(defvar tarfh-list-regexp (concat "([a-zA-Z-]+)\\s+(\\w+)/(\\w+)\\s+(\\d+)\\s+"
-				  "([0-9-]+\\s+[0-9:]+)\\s+([^\n]+)"))
+(defvar tarfh-list-regexp (concat "([a-zA-Z-]+)\\s+(\\w+)/([a-zA-Z0-9_,]+)\\s+(\\d+)\\s+"
+				  ;; GNU tar output
+				  "([0-9-]+\\s+[0-9:]+"
+				  ;; solaris tar output
+				  "|\\w\\w\\w\\s+\\d+\\s+\\d+:\\d+\\s+\\d+)"
+				  "\\s+([^\n]+)"))
 
 ;; Map list file types to symbols
 (defvar tarfh-list-type-alist '((?- . file) (?d . directory)
@@ -82,7 +94,7 @@
 ;; guards tarfh-created file handles
 (define tarfh-fh-guardian (make-guardian))
 
-
+
 ;; Interface to tar program
 
 (defun tarfh-check-tar-program ()
@@ -102,19 +114,20 @@
     (error "Can't find/execute GNU tar")))
 
 (defun tarfh-call-tar (input-file output op tar-file #!rest args)
-  (unless tarfh-gnu-tar-version
-    (tarfh-check-tar-program))
   ;; XXX handle non-local files by copying
   ;; XXX but then again, that's a bad idea in gaolled code..
-  (setq tar-file (local-file-name tar-file))
-  (let*
-      ((process (make-process output))
-       (mode (cdr (assoc-regexp tar-file tarfh-compression-modes)))
-       (all-args `(,op ,@(and mode (list mode)) "--file" ,tar-file ,@args)))
-    (zerop (apply call-process process input-file
-		  tarfh-gnu-tar-program all-args))))
+  (when (file-exists-p tar-file)
+    (setq tar-file (local-file-name tar-file))
+    (unless tarfh-gnu-tar-version
+      (tarfh-check-tar-program))
+    (let* ((process (make-process output))
+	   (mode (cdr (assoc-regexp tar-file tarfh-compression-modes)))
+	   (all-args `(,op ,@(and mode (list mode))
+		       "--file" ,tar-file ,@args)))
+      (zerop (apply call-process process input-file
+		    tarfh-gnu-tar-program all-args)))))
+
 
-
 ;; extracting files (with caching)
 
 (define cached-file nil)		;name of file
@@ -153,7 +166,7 @@
 (add-hook 'idle-hook empty-file-cache)
 (add-hook 'before-exit-hook empty-file-cache)
 
-
+
 ;; directory caching
 
 (defconst tarfh-file-full-name 0)
@@ -193,7 +206,7 @@
       (setq tarfh-pending-output (substring string point)))))
 
 (defun tarfh-parse-list (string point)
-  (when (string-looking-at tarfh-list-regexp string point)
+  (if (string-looking-at tarfh-list-regexp string point)
     (let
 	((mode-string (substring string (match-start 1) (match-end 1)))
 	 (user (substring string (match-start 2) (match-end 2)))
@@ -209,7 +222,8 @@
       (setq file-name (expand-file-name name ""))
       (vector name file-name size modtime
 	      (cdr (assq (aref mode-string 0) tarfh-list-type-alist))
-	      nil mode-string user group symlink))))
+	      nil mode-string user group symlink))
+	      (error "can't parse tar file listing line (GNU or Solaris tar required): %s" (substring string point))))
 
 (defun tarfh-file-get-modtime (file-struct)
   (when (stringp (aref file-struct tarfh-file-modtime))
@@ -347,7 +361,7 @@
 
 (add-hook 'after-gc-hook tarfh-after-gc)
 
-
+
 ;; file handler
 
 (defun tarfh-split-filename (name)
@@ -387,7 +401,7 @@
 	   (tarfh-handler (car split) (cdr split) op args)))
 	(t
 	 (error "Unknown file op in TAR handler: %s %S" op args))))
-	
+
 (defun tarfh-handler (tarfile rel-file op args)
   (cond
    ((eq op 'canonical-file-name)
